package edu.gac.max.puzzle;

import java.awt.BorderLayout;
import java.awt.EventQueue;
import java.awt.GridLayout;
import java.awt.HeadlessException;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JPanel;

/**
 * @author Max Hailperin, Barbara Kaiser, and Karl Knight
 * 
 * This file contains excerpts from the textbook Concrete Abstractions: An Introduction to Computer Science Using Scheme,
 * by Max Hailperin, Barbara Kaiser, and Karl Knight, Copyright (c) 1998 by the authors. Full text is available for free
 * at http://www.gustavus.edu/+max/concrete-abstractions.html
 * 
 * Modified by Karl Knight in May 2011 to use Java Swing
 * Modified by Max Hailperin in November 2013 to separate the model.
 */
public class GUI extends JFrame implements ModelListener {
	
	// Each tile is an approximate square of approximately this size in pixels
	public static final int BUTTON_WIDTH = 60;
	
	// Computed dimensions of the JFrame
	public static final int WINDOW_WIDTH = Puzzle.SIZE * BUTTON_WIDTH;
	public static final int WINDOW_HEIGHT = (Puzzle.SIZE + 1) * BUTTON_WIDTH;

	private JButton[][] tiles;

	private Model model;
	
	public GUI(Model model) throws HeadlessException {
		super();

		this.model = model;
		
		int numButtons = Puzzle.SIZE * Puzzle.SIZE;
		setTitle((numButtons - 1) + " Puzzle");
		setSize(WINDOW_WIDTH, WINDOW_HEIGHT);

		// controlPanel contains high-level controls for the game
		JPanel controlPanel = new JPanel();
		add(controlPanel, BorderLayout.NORTH);
		
		// Initialize Button, the only high-level control so far
		JButton initializeButton = new JButton("Initialize");
		initializeButton.addActionListener(new InitializeActionListener(model));
		controlPanel.add(initializeButton);
		
		// mainPanel contains the grid of tiles (which are JButtons)
		JPanel mainPanel = new JPanel(new GridLayout(Puzzle.SIZE, Puzzle.SIZE));
		tiles = new JButton[Puzzle.SIZE][Puzzle.SIZE];
		add(mainPanel, BorderLayout.CENTER);
		
		// set up the grid of tiles in mainPanel
		for (int row = 0; row < Puzzle.SIZE; row++) {
			for (int col = 0; col < Puzzle.SIZE; col++) {
				JButton tile = new JButton();
				tiles[row][col] = tile;
				tile.addActionListener(new TileActionListener(model, row, col));
				mainPanel.add(tile);
			}
		}
	}
	
	public void onModelChanged(Model model){
		for(int row = 0; row < Puzzle.SIZE; row++){
			for(int col = 0; col < Puzzle.SIZE; col++){
				tiles[row][col].setText(model.get(row, col));
			}
		}
	}

}
