package edu.gac.max.puzzle;

/**
 * @author Max Hailperin, Barbara Kaiser, and Karl Knight
 * 
 * This file contains excerpts from the textbook Concrete Abstractions: An Introduction to Computer Science Using Scheme,
 * by Max Hailperin, Barbara Kaiser, and Karl Knight, Copyright (c) 1998 by the authors. Full text is available for free
 * at http://www.gustavus.edu/+max/concrete-abstractions.html
 * 
 * Modified by Max Hailperin in November 2013 to separate model from GUI.
 */
public class ModelImpl implements Model {
	
	private String[][] tiles;
	private int blankRow;
	private int blankCol;

	public ModelImpl(){
		tiles = new String[Puzzle.SIZE][Puzzle.SIZE];
		initializeTiles();
	}
	
	public void initializeTiles(){
		int tileCount = 0;
		for(int row = 0; row < Puzzle.SIZE; row++){
			for(int col = 0; col < Puzzle.SIZE; col++){
				tileCount++;
				tiles[row][col] = "" + tileCount;
			}
		}
		blankRow = Puzzle.SIZE - 1;
		blankCol = Puzzle.SIZE - 1;
		tiles[blankRow][blankCol] = "";
		modelChanged();
	}

	public void pushTile(int row, int col){
		if (row == blankRow) {
			for ( ; blankCol < col; blankCol++) {
				tiles[blankRow][blankCol] = tiles[blankRow][blankCol+1];
			}
			for ( ; blankCol > col; blankCol--) {
				tiles[blankRow][blankCol] = tiles[blankRow][blankCol-1];
			}
		} else if (col == blankCol) {
			for ( ; blankRow < row; blankRow++) {
				tiles[blankRow][blankCol] = tiles[blankRow+1][blankCol];
			}
			for ( ; blankRow > row; blankRow--) {
				tiles[blankRow][blankCol] = tiles[blankRow-1][blankCol];
			}
		}
		tiles[blankRow][blankCol] = "";
		modelChanged();
	}

	public String get(int row, int col){
		return tiles[row][col];
	}

	private ModelListener listener;
	
	public void setModelListener(ModelListener listener){
		this.listener = listener;
		listener.onModelChanged(this);
	}

	private void modelChanged(){
		if(listener != null){
			listener.onModelChanged(this);
		}
	}

}
