/**
 * @author Max Hailperin, Barbara Kaiser, and Karl Knight
 * 
 * This file contains excerpts from the textbook Concrete Abstractions: An Introduction to Computer Science Using Scheme,
 * by Max Hailperin, Barbara Kaiser, and Karl Knight, Copyright (c) 1998 by the authors. Full text is available for free
 * at http://www.gustavus.edu/+max/concrete-abstractions.html
 * 
 * Modified by Karl Knight in May 2011 to use Java Swing
 * Modified by Max Hailperin in November 2013 to separate the model and offer JMS connectivity.
 */

package edu.gac.max.puzzle;

import java.awt.EventQueue;
import javax.swing.JFrame;
import javax.annotation.Resource;
import javax.jms.ConnectionFactory;
import javax.jms.JMSContext;
import javax.jms.JMSRuntimeException;
import javax.jms.Destination;

public class Puzzle {
	
	// The tiles of the puzzle are a square grid of Puzzle.SIZE by Puzzle.SIZE tiles
	public static final int SIZE = 4; // should ideally be externally configurable as a property

	@Resource(lookup = "java:comp/DefaultJMSConnectionFactory")
	private static ConnectionFactory connectionFactory;
	@Resource(lookup = "jms/MyQueue")
	private static Destination commands;
	@Resource(lookup = "jms/MyTopic")
	private static Destination states;

	public static void main(String[] args) {
		if(args.length == 1 && args[0].equals("client")){
			EventQueue.invokeLater(new Runnable()
				{
					public void run() {
						try {
							JMSContext context =
								connectionFactory.createContext();
							ModelProxy model =
								new ModelProxy(commands, states, context);
							GUI frame = new GUI(model);
							model.setModelListener(frame);
							frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
							frame.setVisible(true);
						} catch (JMSRuntimeException e) {
							System.err.println("Exception occurred: " + e.toString());
							System.exit(1);
						}
					}
				});
		} else if(args.length == 1 && args[0].equals("server")){
			try (JMSContext context = connectionFactory.createContext();) {
				Model model = new ModelImpl();
				Listener listener = new Listener(model, commands, states, context);
				model.setModelListener(listener);
				System.out.println("Puzzle server running. Press return to terminate.");
				System.in.read();
			} catch (Exception e) {
				System.err.println("Exception occurred: " + e.toString());
				System.exit(1);
			}
		} else {
			System.out.println("Running standalone.");
			EventQueue.invokeLater(new Runnable()
				{
					public void run() {
						Model model = new ModelImpl();
						GUI frame = new GUI(model);
						model.setModelListener(frame);
						frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
						frame.setVisible(true);
					}
				});
		}
	}
}
